/*
 ============================================================================
 Name        : iotest.c
 Author      : dgoldstein
 Version     :
 Copyright   : (c) idesis GmbH 2010
 Description :
 ============================================================================
 */

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <fcntl.h>
#include <unistd.h>
#include <pthread.h>
#include "iotest.h"

pthread_t thread;
pthread_mutex_t logMutex;
clock_t progStart;
char *path;
char *fileName;

int main(int argc, char *argv[]) {
	if (argc > 1) {
		path = argv[1];
	} else {
		path = DEFAULT_PATH;
	}
	fileName = (char *) malloc(strlen(path) + strlen(FILE_NAME) + 1);
	strcpy(fileName, path);
	strcat(fileName, FILE_NAME);
	logger("Writing %ld bytes in %s file", FILE_SIZE, fileName);
	pthread_mutex_init(&logMutex, NULL);
	progStart = clock();
	startThread();
	writeFile();
	stopThread();
	unlink(FILE_NAME);
	pthread_mutex_destroy(&logMutex);
	return EXIT_SUCCESS;
}

void startThread() {
	pthread_create(&thread, NULL, &runner, NULL);
}

void stopThread() {
	pthread_cancel(thread);
}

void *runner(void *arg) {
	int count;
	for (count = 0; ; count++) {
		sleep(1);
		logger("==========> executing (%d) '%s'", count, PROG);
		clock_t start = clock();
		//system(PROG);
		int status;
		pid_t pid = fork();
		if (pid == 0) {
			execv(PROG, NULL);
			exit(0);
		} else if (pid > 0) {
			wait(&status);
		}
		clock_t end = clock();
		double timeellapsed = ((double)(end-start))/CLOCKS_PER_SEC;
		logger("==========> executed (%d) '%s' in %2.3f s", count, PROG, timeellapsed);

	}
	return NULL;
}

void writeFile() {
	char *buffer = (char *) malloc(BUFFER_SIZE);
	ssize_t fileSize = 0;
	int file = open(fileName, O_WRONLY | O_CREAT | O_TRUNC);
	while (fileSize < FILE_SIZE) {
		logger("writing %d bytes", BUFFER_SIZE);
		ssize_t writen = write(file, buffer, BUFFER_SIZE);
		fileSize += writen;
		logger("wrote %d bytes", BUFFER_SIZE);
	}
	close(file);
	free(buffer);
}

void logger(char *format, ...) {
	pthread_mutex_lock(&logMutex);
	va_list args;
	va_start(args, format);
	double timeellapsed = ((double)(clock()-progStart))/CLOCKS_PER_SEC;
	fprintf(stderr, "[ioTestOut] %5.6f: ", timeellapsed);
	vfprintf(stderr, format, args);
	fprintf(stderr, "\n");
	va_end(args);
	fflush(stderr);
	pthread_mutex_unlock(&logMutex);
}
